import unittest
from string import split
from base64 import urlsafe_b64decode, urlsafe_b64encode

def encodePKString(ids):
    """ encode the tuple of PrimaryKeys
        into a dataset of the following structure
        (1,'klaus') --> int:1;str:klaus; """
    pkstring=""
    for key in ids:
        if isinstance(key, int):
	    pkstring += "int:%i;" %key
	elif isinstance(key, str):
	    pkstring += "str:%s;" %key
    return urlsafe_b64encode(pkstring)

def decodePKString(s):
    """ docode the PrimaryKey String back
        into the tuple
        int:1;str:klaus; --> (1,'klaus') """
    s = urlsafe_b64decode(str(s))
    if s.endswith(';'):
        s=s[:-1]
    rc=[]
    for key in split(s,';'):
        typ, value = split(key,':')
        if typ == 'int':
		    rc.append(int(value))
        elif typ == 'str':
		    rc.append(str(value))
    return tuple(rc)

#class Utils(unittest.TestCase):
#    ids = (1,'klaus')
#    string = "int:1;str:klaus;"
#	
#    def test_encodePKString(self):
#        self.assertEqual(encodePKString(self.ids), self.string)
#    
 #   def test_decodePKString(self):
##		self.assertEqual(decodePKString(self.string), self.ids)
#
#if __name__ == "__main__":
#	unittest.main()



import unittest
from zope.testing import doctest
from storm.locals import *

class Person(object):
    __storm_table__ = "person"

    id = Int(primary=True)
    name = Unicode()

def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('README.txt',
			     package="nva.stormcontainer",
                             optionflags=doctest.ELLIPSIS),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



import unittest
from nva.stormcontainer.utils import *
from base64 import urlsafe_b64encode
from base64 import urlsafe_b64decode

class TestUtils(unittest.TestCase):
    ids = (1,'klaus')
    string = "int:1;str:klaus;"
    encode_string = urlsafe_b64encode(string)


    oneids = (5,)
    onestring = "int:5;"
    oneencode_string = urlsafe_b64encode(onestring)


    def test_encodePKString(self):
        self.assertEqual(encodePKString(self.ids), self.encode_string)

    def test_decodePKString(self):
        self.assertEqual(decodePKString(self.encode_string), self.ids)

    def test_oneencodePKString(self):
        self.assertEqual(encodePKString(self.oneids), self.oneencode_string)

    def test_onedecodePKString(self):
        self.assertEqual(decodePKString(self.oneencode_string), self.oneids)



def test_suite():
    return unittest.TestSuite( unittest.makeSuite(TestUtils),)

if __name__ == "__main__":
        unittest.main()




from string import split

import transaction

from zope import interface
from zope.component import getUtility
from zope.configuration.name import resolve
from zope.interface import directlyProvides, directlyProvidedBy

from zope.location.interfaces import ILocation
from zope.exceptions.interfaces import UserError
from zope.security.proxy import removeSecurityProxy
from zope.proxy import ProxyBase, sameProxiedObjects

from zope.app.container.contained import Contained
from zope.app.container.contained import NameChooser
from zope.app.container.interfaces import IContained
from zope.app.container.contained import ContainedProxy

from storm.zope.interfaces import IZStorm
from storm.info import get_obj_info

from interfaces import IStormContainer
from nva.stormcontainer.utils import *

def contained(obj, parent=None, name=None):
    """An implementation of zope.app.container.contained.contained
    that doesn't generate events, for internal use.

    Borrowed from SQLOS.
    """
    if (parent is None):
        raise TypeError, 'Must provide a parent'

    if not IContained.providedBy(obj):
        if ILocation.providedBy(obj):
            directlyProvides(obj, IContained, directlyProvidedBy(obj))
        else:
            obj = ContainedProxy(obj)

    oldparent = obj.__parent__
    oldname = obj.__name__

    if (oldparent is None) or not (oldparent is parent
                                   or sameProxiedObjects(oldparent, parent)):
        obj.__parent__ = parent

    if oldname != name and name is not None:
        obj.__name__ = name

    return obj


#class StormNameChooser(NameChooser):
#
#    def checkName(self, name, content):
#        if isinstance(name, str):
#            name = unicode(name)
#        elif not isinstance(name, unicode):
#            raise TypeError("Invalid name type", type(name))
#
#        unproxied = removeSecurityProxy(self.context)
#        if not name.startswith(unproxied._class.__name__+'-'):
#            raise UserError("Invalid name for Storm object")
#        return True
#
#    def chooseName(self, name, obj):
#        # flush the object to make sure it contains an id
#        session = z3c.zalchemy.getSession()
#        session.save(obj)
#        session.flush()
#        return self.context._toStringIdentifier(obj)

class StormContainer(object):
    """ Make a Storm Container for Zope"""
    interface.implements(IStormContainer)

    def __init__(self):
        super(StormContainer,self).__init__()

        _className = ''
        _class = None
        _storeUtilityName = ''

    def setStoreUtilityName(self, name):
        self._storeUtilityName = name

    def getStoreUtilityName(self):
        return self._storeUtilityName

    storeUtilityName = property(getStoreUtilityName, setStoreUtilityName) 

    def setClassName(self, name):
        self._className = name
        self._class=resolve(name)

    def getClassName(self):
        return self._className

    className = property(getClassName, setClassName)

    def keys(self):
        for name, obj in self.items():
            yield name

    def values(self):
        for name, obj in self.items():
            yield obj

    def __iter__(self):
        return iter(self.keys())

    def items(self):
        store = getUtility(IZStorm).get(self.getStoreUtilityName())
        for obj in store.find(self._class):
            name = self._toStringIdentifier(obj)
            yield (name, contained(obj, self, name) )

    def __getitem__(self, name):
        if not isinstance(name, basestring):
            raise KeyError, "%s is not a string" % name
        obj = self._fromStringIdentifier(name)
        if obj is None:
            raise KeyError(name)
        return contained(obj, self, name)

    def get(self, name, default=None):
        try:
            return self[name]
        except KeyError:
            return default
    
    def __contains__(self, name):
        return self.get(name) is not None

    def __len__(self):
        try:
            store = getUtility(IZStorm).get(self.getStoreUtilityName())
	    query = store.find(self._class)
            return query.count()
        except:
            # we don't want an exception in case of database problems
            return 0

    def __delitem__(self, name):
        obj = self._fromStringIdentifier(name) 
        #TODO: better delete objects using a delete adapter
        #      for dependency handling.
        store = getUtility(IZStorm).get(self.getStoreUtilityName())
        store.remove(obj)
        transaction.commit()
        #store.flush()
	    #store.commit()

    def __setitem__(self, name, item):
        store = getUtility(IZStorm).get(self.getStoreUtilityName())
        store.add(item)
        transaction.commit()
        #store.flush()
	    #store.commit()

    def _toStringIdentifier(self, obj):
        store = getUtility(IZStorm).get(self.getStoreUtilityName())
        ident = self._getInstanceKey(obj)
        prefix = "%s-" %self._class.__name__
        return "%s%s" %(prefix, ident)

    def _fromStringIdentifier(self, name):
        prefix = "%s-" %self._class.__name__
        if not str(name).startswith(prefix):
            return None
        store = getUtility(IZStorm).get(self.getStoreUtilityName())
        id = name[ len(str(self._class.__name__)) + 1: ]
        key = decodePKString(id)
        return store.get(self._class, key)

    def _getInstanceKey(self, obj):
        """ Get an unique id form obj """
        obj_info = get_obj_info(obj)
        ids = tuple(obj_info.variables[column].get() for column in obj_info.cls_info.primary_key)
	return encodePKString(ids)



from setuptools import setup, find_packages

version = '0.2'

long_description = "A Zope container for Storm" 

setup(name='nva.stormcontainer',
      version=version,
      description="A Zope Storm Container",
      long_description=long_description,
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Framework :: Zope3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='Storm Zope',
      author="'Christian Klinger'",
      author_email='cklinger@novareto.de',
      url='http://www.novareto.de',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['nva'],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'storm',
                        'ZODB3',
                        'zope.component',
                        'zope.interface',
                        'zope.schema',
                        'zope.app.testing',
                        'zope.app.component',
                        'zope.app.keyreference',
                        'zope.app.container',
                        'zope.app.pagetemplate',
                        'pysqlite',
                       ],      
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
